# Stub for tabulate: https://bitbucket.org/astanin/python-tabulate
from typing import Any, Dict, Iterable, Sequence, Union


def __getattr__(name: str) -> Any: ...

def tabulate(
    tabular_data: Iterable[Iterable[Any]],
    headers: Union[str, Dict[str, str], Sequence[str]] = ...,
    tablefmt: str = ...,
    floatfmt: str = ...,
    numalign: str = ...,
    stralign: str = ...,
    missingval: str = ...,
    showindex: str = ...,
    disable_numparse: bool = ...
) -> str:
    ...



from typing import Any, Dict, Optional, Tuple
from tkinter import Button, commondialog, Entry, Frame, Listbox, Scrollbar, Toplevel

dialogstates: Dict[Any, Tuple[Any, Any]]

class FileDialog:
    title: str = ...
    master: Any = ...
    directory: Optional[Any] = ...
    top: Toplevel = ...
    botframe: Frame = ...
    selection: Entry = ...
    filter: Entry = ...
    midframe: Entry = ...
    filesbar: Scrollbar = ...
    files: Listbox = ...
    dirsbar: Scrollbar = ...
    dirs: Listbox = ...
    ok_button: Button = ...
    filter_button: Button = ...
    cancel_button: Button = ...
    def __init__(self, master, title: Optional[Any] = ...) -> None: ...  # title is usually a str or None, but e.g. int doesn't raise en exception either
    how: Optional[Any] = ...
    def go(self, dir_or_file: Any = ..., pattern: str = ..., default: str = ..., key: Optional[Any] = ...): ...
    def quit(self, how: Optional[Any] = ...) -> None: ...
    def dirs_double_event(self, event) -> None: ...
    def dirs_select_event(self, event) -> None: ...
    def files_double_event(self, event) -> None: ...
    def files_select_event(self, event) -> None: ...
    def ok_event(self, event) -> None: ...
    def ok_command(self) -> None: ...
    def filter_command(self, event: Optional[Any] = ...) -> None: ...
    def get_filter(self): ...
    def get_selection(self): ...
    def cancel_command(self, event: Optional[Any] = ...) -> None: ...
    def set_filter(self, dir, pat) -> None: ...
    def set_selection(self, file) -> None: ...

class LoadFileDialog(FileDialog):
    title: str = ...
    def ok_command(self) -> None: ...

class SaveFileDialog(FileDialog):
    title: str = ...
    def ok_command(self): ...

class _Dialog(commondialog.Dialog): ...

class Open(_Dialog):
    command: str = ...

class SaveAs(_Dialog):
    command: str = ...

class Directory(commondialog.Dialog):
    command: str = ...

def askopenfilename(**options): ...
def asksaveasfilename(**options): ...
def askopenfilenames(**options): ...
def askopenfile(mode: str = ..., **options): ...
def askopenfiles(mode: str = ..., **options): ...
def asksaveasfile(mode: str = ..., **options): ...
def askdirectory(**options): ...
def test() -> None: ...



from typing import Any

def is_known_charset(charset): ...

class JSONRequestMixin:
    def json(self): ...

class ProtobufRequestMixin:
    protobuf_check_initialization: Any
    def parse_protobuf(self, proto_type): ...

class RoutingArgsRequestMixin:
    routing_args: Any
    routing_vars: Any

class ReverseSlashBehaviorRequestMixin:
    def path(self): ...
    def script_root(self): ...

class DynamicCharsetRequestMixin:
    default_charset: Any
    def unknown_charset(self, charset): ...
    def charset(self): ...

class DynamicCharsetResponseMixin:
    default_charset: Any
    charset: Any



# Stubs for six.moves.urllib.parse
#
# Note: Commented out items means they weren't implemented at the time.
# Uncomment them when the modules have been added to the typeshed.
from urllib.parse import ParseResult as ParseResult
from urllib.parse import SplitResult as SplitResult
from urllib.parse import parse_qs as parse_qs
from urllib.parse import parse_qsl as parse_qsl
from urllib.parse import urldefrag as urldefrag
from urllib.parse import urljoin as urljoin
from urllib.parse import urlparse as urlparse
from urllib.parse import urlsplit as urlsplit
from urllib.parse import urlunparse as urlunparse
from urllib.parse import urlunsplit as urlunsplit
from urllib.parse import quote as quote
from urllib.parse import quote_plus as quote_plus
from urllib.parse import unquote as unquote
from urllib.parse import unquote_plus as unquote_plus
from urllib.parse import urlencode as urlencode
# from urllib.parse import splitquery as splitquery
# from urllib.parse import splittag as splittag
# from urllib.parse import splituser as splituser
from urllib.parse import uses_fragment as uses_fragment
from urllib.parse import uses_netloc as uses_netloc
from urllib.parse import uses_params as uses_params
from urllib.parse import uses_query as uses_query
from urllib.parse import uses_relative as uses_relative



"""Stub file for the '_functools' module."""

from typing import Any, Callable, Dict, Iterable, Optional, TypeVar, Tuple, overload

_T = TypeVar("_T")
_T2 = TypeVar("_T2")
_T3 = TypeVar("_T3")
_T4 = TypeVar("_T4")
_T5 = TypeVar("_T5")
_S = TypeVar("_S")

@overload
def reduce(function: Callable[[_T, _T], _T],
           sequence: Iterable[_T]) -> _T: ...
@overload
def reduce(function: Callable[[_T, _S], _T],
           sequence: Iterable[_S], initial: _T) -> _T: ...

@overload
def partial(__func: Callable[[_T], _S], __arg: _T) -> Callable[[], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2], _S], __arg: _T) -> Callable[[_T2], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3], _S], __arg: _T) -> Callable[[_T2, _T3], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4], _S], __arg: _T) -> Callable[[_T2, _T3, _T4], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4, _T5], _S], __arg: _T) -> Callable[[_T2, _T3, _T4, _T5], _S]: ...

@overload
def partial(__func: Callable[[_T, _T2], _S],
            __arg1: _T,
            __arg2: _T2) -> Callable[[], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3], _S],
            __arg1: _T,
            __arg2: _T2) -> Callable[[_T3], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4], _S],
            __arg1: _T,
            __arg2: _T2) -> Callable[[_T3, _T4], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4, _T5], _S],
            __arg1: _T,
            __arg2: _T2) -> Callable[[_T3, _T4, _T5], _S]: ...

@overload
def partial(__func: Callable[[_T, _T2, _T3], _S],
            __arg1: _T,
            __arg2: _T2,
            __arg3: _T3) -> Callable[[], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4], _S],
            __arg1: _T,
            __arg2: _T2,
            __arg3: _T3) -> Callable[[_T4], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4, _T5], _S],
            __arg1: _T,
            __arg2: _T2,
            __arg3: _T3) -> Callable[[_T4, _T5], _S]: ...

@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4], _S],
            __arg1: _T,
            __arg2: _T2,
            __arg3: _T3,
            __arg4: _T4) -> Callable[[], _S]: ...
@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4, _T5], _S],
            __arg1: _T,
            __arg2: _T2,
            __arg3: _T3,
            __arg4: _T4) -> Callable[[_T5], _S]: ...

@overload
def partial(__func: Callable[[_T, _T2, _T3, _T4, _T5], _S],
            __arg1: _T,
            __arg2: _T2,
            __arg3: _T3,
            __arg4: _T4,
            __arg5: _T5) -> Callable[[], _S]: ...

@overload
def partial(__func: Callable[..., _S],
            *args: Any,
            **kwargs: Any) -> Callable[..., _S]: ...



import time
import random

import graphics as g


class Car(g.shapes.Image):

    def __init__(self):
        super(Car, self).__init__()

    def genImage(self):
        return [[0, 1, 0], [0, 0, 0], [0, 1, 0]]


def main():

    FPS = 15
    screen = g.Canvas(size=(36, 20))

    touchMeter = g.Sprite(g.shapes.Text(''), color=g.colors.RED)
    mover = g.Sprite(
        Car(),
        color=g.colors.WHITE
    )

    gridImg = ([[False, True, False, False] * 9] + ([[False] * 36]) * 3) * 5
    grid = g.Sprite(
        g.shapes.CustImage(gridImg),
        color=g.colors.GREEN,
        position=(0, 1)
    )

    screen.sprites.append(touchMeter)
    screen.sprites.append(grid)
    screen.sprites.append(mover)

    frame = 0
    t = time.time()
    with g.NonBlockingInput() as nbi:
        while True:

            ch = nbi.char()
            if ch == '.':
                mover.move(g.RIGHT)
            if ch == ',':
                mover.move(g.LEFT)
            if ch == '/':
                mover.move(g.UP)
            if ch == 'm':
                mover.move(g.DOWN)

            str_ = ''
            if mover.touching(screen, 0):
                str_ += str(0)
            if mover.touching(screen, 1):
                str_ += str(1)
            if mover.touching(screen, 2):
                str_ += str(2)
            if mover.touching(screen, 3):
                str_ += str(3)
            if mover.touching(screen):
                str_ += 'T'

            touchMeter.image.text = str_

            if time.time() >= t + (1 / FPS):
                t = time.time()
                print(screen)

if __name__ == '__main__':
    main()



class CryptoWarning(Warning): ...
class CryptoDeprecationWarning(DeprecationWarning, CryptoWarning): ...
class CryptoRuntimeWarning(RuntimeWarning, CryptoWarning): ...
class RandomPool_DeprecationWarning(CryptoDeprecationWarning): ...
class ClockRewindWarning(CryptoRuntimeWarning): ...
class GetRandomNumber_DeprecationWarning(CryptoDeprecationWarning): ...
class PowmInsecureWarning(CryptoRuntimeWarning): ...
